@@ -15,13 +15,14 @@ available in your version of Python.
 The preferred alias for `defchararray` is `numpy.char`.
 
 """
+from __future__ import division, absolute_import, print_function
 
 import sys
-from numerictypes import string_, unicode_, integer, object_, bool_, character
-from numeric import ndarray, compare_chararrays
-from numeric import array as narray
+from .numerictypes import string_, unicode_, integer, object_, bool_, character
+from .numeric import ndarray, compare_chararrays
+from .numeric import array as narray
 from numpy.core.multiarray import _vec_string
-from numpy.compat import asbytes
+from numpy.compat import asbytes, long
 import numpy
 
 __all__ = ['chararray',
@@ -89,7 +90,7 @@ def _get_num_chars(a):
     for a unicode array this is itemsize / 4.
     """
     if issubclass(a.dtype.type, unicode_):
-        return a.itemsize / 4
+        return a.itemsize // 4
     return a.itemsize
 
 
@@ -259,18 +260,23 @@ def str_len(a):
 
 def add(x1, x2):
     """
-    Return (x1 + x2), that is string concatenation, element-wise for a
-    pair of array_likes of str or unicode.
+    Return element-wise string concatenation for two arrays of str or unicode.
+
+    Arrays `x1` and `x2` must have the same shape.
 
     Parameters
     ----------
     x1 : array_like of str or unicode
+        Input array.
     x2 : array_like of str or unicode
+        Input array.
 
     Returns
     -------
-    out : ndarray
+    add : ndarray
         Output array of `string_` or `unicode_`, depending on input types
+        of the same shape as `x1` and `x2`.
+
     """
     arr1 = numpy.asarray(x1)
     arr2 = numpy.asarray(x2)
@@ -301,7 +307,7 @@ def multiply(a, i):
     a_arr = numpy.asarray(a)
     i_arr = numpy.asarray(i)
     if not issubclass(i_arr.dtype.type, integer):
-        raise ValueError, "Can only multiply by integers"
+        raise ValueError("Can only multiply by integers")
     out_size = _get_num_chars(a_arr) * max(long(i_arr.max()), 0)
     return _vec_string(
         a_arr, (a_arr.dtype.type, out_size), '__mul__', (i_arr,))
@@ -344,6 +350,7 @@ def capitalize(a):
     Parameters
     ----------
     a : array_like of str or unicode
+        Input array of strings to capitalize.
 
     Returns
     -------
@@ -363,73 +370,47 @@ def capitalize(a):
     >>> np.char.capitalize(c)
     array(['A1b2', '1b2a', 'B2a1', '2a1b'],
         dtype='|S4')
+
     """
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'capitalize')
 
-if sys.version_info >= (2, 4):
-    def center(a, width, fillchar=' '):
-        """
-        Return a copy of `a` with its elements centered in a string of
-        length `width`.
 
-        Calls `str.center` element-wise.
+def center(a, width, fillchar=' '):
+    """
+    Return a copy of `a` with its elements centered in a string of
+    length `width`.
 
-        Parameters
-        ----------
-        a : array_like of str or unicode
+    Calls `str.center` element-wise.
 
-        width : int
-            The length of the resulting strings
-        fillchar : str or unicode, optional
-            The padding character to use (default is space).
+    Parameters
+    ----------
+    a : array_like of str or unicode
 
-        Returns
-        -------
-        out : ndarray
-            Output array of str or unicode, depending on input
-            types
+    width : int
+        The length of the resulting strings
+    fillchar : str or unicode, optional
+        The padding character to use (default is space).
 
-        See also
-        --------
-        str.center
+    Returns
+    -------
+    out : ndarray
+        Output array of str or unicode, depending on input
+        types
 
-        """
-        a_arr = numpy.asarray(a)
-        width_arr = numpy.asarray(width)
-        size = long(numpy.max(width_arr.flat))
-        if numpy.issubdtype(a_arr.dtype, numpy.string_):
-            fillchar = asbytes(fillchar)
-        return _vec_string(
-            a_arr, (a_arr.dtype.type, size), 'center', (width_arr, fillchar))
-else:
-    def center(a, width):
-        """
-        Return an array with the elements of `a` centered in a string
-        of length width.
+    See also
+    --------
+    str.center
 
-        Calls `str.center` element-wise.
+    """
+    a_arr = numpy.asarray(a)
+    width_arr = numpy.asarray(width)
+    size = long(numpy.max(width_arr.flat))
+    if numpy.issubdtype(a_arr.dtype, numpy.string_):
+        fillchar = asbytes(fillchar)
+    return _vec_string(
+        a_arr, (a_arr.dtype.type, size), 'center', (width_arr, fillchar))
 
-        Parameters
-        ----------
-        a : array_like of str or unicode
-        width : int
-            The length of the resulting strings
-
-        Returns
-        -------
-        out : ndarray, str or unicode
-            Output array of str or unicode, depending on input types
-
-        See also
-        --------
-        str.center
-        """
-        a_arr = numpy.asarray(a)
-        width_arr = numpy.asarray(width)
-        size = long(numpy.max(width_arr.flat))
-        return _vec_string(
-            a_arr, (a_arr.dtype.type, size), 'center', (width_arr,))
 
 def count(a, sub, start=0, end=None):
     """
@@ -476,6 +457,7 @@ def count(a, sub, start=0, end=None):
     """
     return _vec_string(a, integer, 'count', [sub, start] + _clean_args(end))
 
+
 def decode(a, encoding=None, errors=None):
     """
     Calls `str.decode` element-wise.
@@ -521,6 +503,7 @@ def decode(a, encoding=None, errors=None):
     return _to_string_or_unicode_array(
         _vec_string(a, object_, 'decode', _clean_args(encoding, errors)))
 
+
 def encode(a, encoding=None, errors=None):
     """
     Calls `str.encode` element-wise.
@@ -555,6 +538,7 @@ def encode(a, encoding=None, errors=None):
     return _to_string_or_unicode_array(
         _vec_string(a, object_, 'encode', _clean_args(encoding, errors)))
 
+
 def endswith(a, suffix, start=0, end=None):
     """
     Returns a boolean array which is `True` where the string element
@@ -598,6 +582,7 @@ def endswith(a, suffix, start=0, end=None):
     return _vec_string(
         a, bool_, 'endswith', [suffix, start] + _clean_args(end))
 
+
 def expandtabs(a, tabsize=8):
     """
     Return a copy of each string element where all tab characters are
@@ -608,14 +593,16 @@ def expandtabs(a, tabsize=8):
     Return a copy of each string element where all tab characters are
     replaced by one or more spaces, depending on the current column
     and the given `tabsize`. The column number is reset to zero after
-    each newline occurring in the string. If `tabsize` is not given, a
-    tab size of 8 characters is assumed. This doesn't understand other
+    each newline occurring in the string. This doesn't understand other
     non-printing characters or escape sequences.
 
     Parameters
     ----------
     a : array_like of str or unicode
+        Input array
     tabsize : int, optional
+        Replace tabs with `tabsize` number of spaces.  If not given defaults
+        to 8 spaces.
 
     Returns
     -------
@@ -625,10 +612,12 @@ def expandtabs(a, tabsize=8):
     See also
     --------
     str.expandtabs
+
     """
     return _to_string_or_unicode_array(
         _vec_string(a, object_, 'expandtabs', (tabsize,)))
 
+
 def find(a, sub, start=0, end=None):
     """
     For each element, return the lowest index in the string where
@@ -663,10 +652,6 @@ def find(a, sub, start=0, end=None):
     return _vec_string(
         a, integer, 'find', [sub, start] + _clean_args(end))
 
-# if sys.version_info >= (2.6):
-#     def format(a, *args, **kwargs):
-#         # _vec_string doesn't support kwargs at present
-#         raise NotImplementedError
 
 def index(a, sub, start=0, end=None):
     """
@@ -890,72 +875,45 @@ def join(sep, seq):
     return _to_string_or_unicode_array(
         _vec_string(sep, object_, 'join', (seq,)))
 
-if sys.version_info >= (2, 4):
-    def ljust(a, width, fillchar=' '):
-        """
-        Return an array with the elements of `a` left-justified in a
-        string of length `width`.
 
-        Calls `str.ljust` element-wise.
+def ljust(a, width, fillchar=' '):
+    """
+    Return an array with the elements of `a` left-justified in a
+    string of length `width`.
 
-        Parameters
-        ----------
-        a : array_like of str or unicode
+    Calls `str.ljust` element-wise.
 
-        width : int
-            The length of the resulting strings
-        fillchar : str or unicode, optional
-            The character to use for padding
+    Parameters
+    ----------
+    a : array_like of str or unicode
 
-        Returns
-        -------
-        out : ndarray
-            Output array of str or unicode, depending on input type
+    width : int
+        The length of the resulting strings
+    fillchar : str or unicode, optional
+        The character to use for padding
 
-        See also
-        --------
-        str.ljust
+    Returns
+    -------
+    out : ndarray
+        Output array of str or unicode, depending on input type
 
-        """
-        a_arr = numpy.asarray(a)
-        width_arr = numpy.asarray(width)
-        size = long(numpy.max(width_arr.flat))
-        if numpy.issubdtype(a_arr.dtype, numpy.string_):
-            fillchar = asbytes(fillchar)
-        return _vec_string(
-            a_arr, (a_arr.dtype.type, size), 'ljust', (width_arr, fillchar))
-else:
-    def ljust(a, width):
-        """
-        Return an array with the elements of `a` left-justified in a
-        string of length `width`.
+    See also
+    --------
+    str.ljust
 
-        Calls `str.ljust` element-wise.
+    """
+    a_arr = numpy.asarray(a)
+    width_arr = numpy.asarray(width)
+    size = long(numpy.max(width_arr.flat))
+    if numpy.issubdtype(a_arr.dtype, numpy.string_):
+        fillchar = asbytes(fillchar)
+    return _vec_string(
+        a_arr, (a_arr.dtype.type, size), 'ljust', (width_arr, fillchar))
 
-        Parameters
-        ----------
-        a : array_like of str or unicode
-        width : int
-            The length of the resulting strings
-
-        Returns
-        -------
-        out : ndarray
-            Output array of str or unicode, depending on input type
-
-        See also
-        --------
-        str.ljust
-        """
-        a_arr = numpy.asarray(a)
-        width_arr = numpy.asarray(width)
-        size = long(numpy.max(width_arr.flat))
-        return _vec_string(
-            a_arr, (a_arr.dtype.type, size), 'ljust', (width_arr,))
 
 def lower(a):
     """
-    Return an array with the elements of `a` converted to lowercase.
+    Return an array with the elements converted to lowercase.
 
     Call `str.lower` element-wise.
 
@@ -963,11 +921,12 @@ def lower(a):
 
     Parameters
     ----------
-    a : array-like of str or unicode
+    a : array_like, {str, unicode}
+        Input array.
 
     Returns
     -------
-    out : ndarray, str or unicode
+    out : ndarray, {str, unicode}
         Output array of str or unicode, depending on input type
 
     See also
@@ -982,10 +941,12 @@ def lower(a):
     >>> np.char.lower(c)
     array(['a1b c', '1bca', 'bca1'],
           dtype='|S5')
+
     """
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'lower')
 
+
 def lstrip(a, chars=None):
     """
     For each element in `a`, return a copy with the leading characters
@@ -995,18 +956,19 @@ def lstrip(a, chars=None):
 
     Parameters
     ----------
-    a : array-like of str or unicode
+    a : array-like, {str, unicode}
+        Input array.
 
-    chars : str or unicode, optional
-       The `chars` argument is a string specifying the set of
-       characters to be removed. If omitted or None, the `chars`
-       argument defaults to removing whitespace. The `chars` argument
-       is not a prefix; rather, all combinations of its values are
-       stripped.
+    chars : {str, unicode}, optional
+        The `chars` argument is a string specifying the set of
+        characters to be removed. If omitted or None, the `chars`
+        argument defaults to removing whitespace. The `chars` argument
+        is not a prefix; rather, all combinations of its values are
+        stripped.
 
     Returns
     -------
-    out : ndarray, str or unicode
+    out : ndarray, {str, unicode}
         Output array of str or unicode, depending on input type
 
     See also
@@ -1019,9 +981,14 @@ def lstrip(a, chars=None):
     >>> c
     array(['aAaAaA', '  aA  ', 'abBABba'],
         dtype='|S7')
-    >>> np.char.lstrip(c, 'a') # 'a' unstripped from c[1] because whitespace leading
+
+    The 'a' variable is unstripped from c[1] because whitespace leading.
+
+    >>> np.char.lstrip(c, 'a')
     array(['AaAaA', '  aA  ', 'bBABba'],
         dtype='|S7')
+
+
     >>> np.char.lstrip(c, 'A') # leaves c unchanged
     array(['aAaAaA', '  aA  ', 'abBABba'],
         dtype='|S7')
@@ -1036,37 +1003,41 @@ def lstrip(a, chars=None):
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'lstrip', (chars,))
 
-if sys.version_info >= (2, 5):
-    def partition(a, sep):
-        """
-        Partition each element in `a` around `sep`.
 
-        Calls `str.partition` element-wise.
+def partition(a, sep):
+    """
+    Partition each element in `a` around `sep`.
 
-        For each element in `a`, split the element as the first
-        occurrence of `sep`, and return 3 strings containing the part
-        before the separator, the separator itself, and the part after
-        the separator. If the separator is not found, return 3 strings
-        containing the string itself, followed by two empty strings.
+    Calls `str.partition` element-wise.
 
-        Parameters
-        ----------
-        a : array-like of str or unicode
-        sep : str or unicode
+    For each element in `a`, split the element as the first
+    occurrence of `sep`, and return 3 strings containing the part
+    before the separator, the separator itself, and the part after
+    the separator. If the separator is not found, return 3 strings
+    containing the string itself, followed by two empty strings.
 
-        Returns
-        -------
-        out : ndarray
-            Output array of str or unicode, depending on input type.
-            The output array will have an extra dimension with 3
-            elements per input element.
+    Parameters
+    ----------
+    a : array_like, {str, unicode}
+        Input array
+    sep : {str, unicode}
+        Separator to split each string element in `a`.
+
+    Returns
+    -------
+    out : ndarray, {str, unicode}
+        Output array of str or unicode, depending on input type.
+        The output array will have an extra dimension with 3
+        elements per input element.
+
+    See also
+    --------
+    str.partition
+
+    """
+    return _to_string_or_unicode_array(
+        _vec_string(a, object_, 'partition', (sep,)))
 
-        See also
-        --------
-        str.partition
-        """
-        return _to_string_or_unicode_array(
-            _vec_string(a, object_, 'partition', (sep,)))
 
 def replace(a, old, new, count=None):
     """
@@ -1099,6 +1070,7 @@ def replace(a, old, new, count=None):
         _vec_string(
             a, object_, 'replace', [old, new] +_clean_args(count)))
 
+
 def rfind(a, sub, start=0, end=None):
     """
     For each element in `a`, return the highest index in the string
@@ -1130,6 +1102,7 @@ def rfind(a, sub, start=0, end=None):
     return _vec_string(
         a, integer, 'rfind', [sub, start] + _clean_args(end))
 
+
 def rindex(a, sub, start=0, end=None):
     """
     Like `rfind`, but raises `ValueError` when the substring `sub` is
@@ -1158,137 +1131,113 @@ def rindex(a, sub, start=0, end=None):
     return _vec_string(
         a, integer, 'rindex', [sub, start] + _clean_args(end))
 
-if sys.version_info >= (2, 4):
-    def rjust(a, width, fillchar=' '):
-        """
-        Return an array with the elements of `a` right-justified in a
-        string of length `width`.
 
-        Calls `str.rjust` element-wise.
+def rjust(a, width, fillchar=' '):
+    """
+    Return an array with the elements of `a` right-justified in a
+    string of length `width`.
 
-        Parameters
-        ----------
-        a : array_like of str or unicode
+    Calls `str.rjust` element-wise.
 
-        width : int
-            The length of the resulting strings
-        fillchar : str or unicode, optional
-            The character to use for padding
+    Parameters
+    ----------
+    a : array_like of str or unicode
 
-        Returns
-        -------
-        out : ndarray
-            Output array of str or unicode, depending on input type
+    width : int
+        The length of the resulting strings
+    fillchar : str or unicode, optional
+        The character to use for padding
 
-        See also
-        --------
-        str.rjust
+    Returns
+    -------
+    out : ndarray
+        Output array of str or unicode, depending on input type
 
-        """
-        a_arr = numpy.asarray(a)
-        width_arr = numpy.asarray(width)
-        size = long(numpy.max(width_arr.flat))
-        if numpy.issubdtype(a_arr.dtype, numpy.string_):
-            fillchar = asbytes(fillchar)
-        return _vec_string(
-            a_arr, (a_arr.dtype.type, size), 'rjust', (width_arr, fillchar))
-else:
-    def rjust(a, width):
-        """
-        Return an array with the elements of `a` right-justified in a
-        string of length `width`.
+    See also
+    --------
+    str.rjust
 
-        Calls `str.rjust` element-wise.
+    """
+    a_arr = numpy.asarray(a)
+    width_arr = numpy.asarray(width)
+    size = long(numpy.max(width_arr.flat))
+    if numpy.issubdtype(a_arr.dtype, numpy.string_):
+        fillchar = asbytes(fillchar)
+    return _vec_string(
+        a_arr, (a_arr.dtype.type, size), 'rjust', (width_arr, fillchar))
 
-        Parameters
-        ----------
-        a : array_like of str or unicode
-        width : int
-            The length of the resulting strings
 
-        Returns
-        -------
-        out : ndarray
-            Output array of str or unicode, depending on input type
+def rpartition(a, sep):
+    """
+    Partition (split) each element around the right-most separator.
 
-        See also
-        --------
-        str.rjust
-        """
-        a_arr = numpy.asarray(a)
-        width_arr = numpy.asarray(width)
-        size = long(numpy.max(width_arr.flat))
-        return _vec_string(
-            a_arr, (a_arr.dtype.type, size), 'rjust', (width,))
+    Calls `str.rpartition` element-wise.
 
-if sys.version_info >= (2, 5):
-    def rpartition(a, sep):
-        """
-        Partition each element in `a` around `sep`.
+    For each element in `a`, split the element as the last
+    occurrence of `sep`, and return 3 strings containing the part
+    before the separator, the separator itself, and the part after
+    the separator. If the separator is not found, return 3 strings
+    containing the string itself, followed by two empty strings.
 
-        Calls `str.rpartition` element-wise.
+    Parameters
+    ----------
+    a : array_like of str or unicode
+        Input array
+    sep : str or unicode
+        Right-most separator to split each element in array.
 
-        For each element in `a`, split the element as the last
-        occurrence of `sep`, and return 3 strings containing the part
-        before the separator, the separator itself, and the part after
-        the separator. If the separator is not found, return 3 strings
-        containing the string itself, followed by two empty strings.
+    Returns
+    -------
+    out : ndarray
+        Output array of string or unicode, depending on input
+        type.  The output array will have an extra dimension with
+        3 elements per input element.
 
-        Parameters
-        ----------
-        a : array-like of str or unicode
-        sep : str or unicode
+    See also
+    --------
+    str.rpartition
 
-        Returns
-        -------
-        out : ndarray
-            Output array of string or unicode, depending on input
-            type.  The output array will have an extra dimension with
-            3 elements per input element.
+    """
+    return _to_string_or_unicode_array(
+        _vec_string(a, object_, 'rpartition', (sep,)))
 
-        See also
-        --------
-        str.rpartition
-        """
-        return _to_string_or_unicode_array(
-            _vec_string(a, object_, 'rpartition', (sep,)))
 
-if sys.version_info >= (2, 4):
-    def rsplit(a, sep=None, maxsplit=None):
-        """
-        For each element in `a`, return a list of the words in the
-        string, using `sep` as the delimiter string.
+def rsplit(a, sep=None, maxsplit=None):
+    """
+    For each element in `a`, return a list of the words in the
+    string, using `sep` as the delimiter string.
 
-        Calls `str.rsplit` element-wise.
+    Calls `str.rsplit` element-wise.
 
-        Except for splitting from the right, `rsplit`
-        behaves like `split`.
+    Except for splitting from the right, `rsplit`
+    behaves like `split`.
 
-        Parameters
-        ----------
-        a : array_like of str or unicode
+    Parameters
+    ----------
+    a : array_like of str or unicode
 
-        sep : str or unicode, optional
-            If `sep` is not specified or `None`, any whitespace string
-            is a separator.
-        maxsplit : int, optional
-            If `maxsplit` is given, at most `maxsplit` splits are done,
-            the rightmost ones.
+    sep : str or unicode, optional
+        If `sep` is not specified or `None`, any whitespace string
+        is a separator.
+    maxsplit : int, optional
+        If `maxsplit` is given, at most `maxsplit` splits are done,
+        the rightmost ones.
 
-        Returns
-        -------
-        out : ndarray
-           Array of list objects
+    Returns
+    -------
+    out : ndarray
+       Array of list objects
 
-        See also
-        --------
-        str.rsplit, split
+    See also
+    --------
+    str.rsplit, split
+
+    """
+    # This will return an array of lists of different sizes, so we
+    # leave it as an object array
+    return _vec_string(
+        a, object_, 'rsplit', [sep] + _clean_args(maxsplit))
 
-        """
-        # This will return an array of lists of different sizes, so we
-        # leave it as an object array
-        return _vec_string(
-            a, object_, 'rsplit', [sep] + _clean_args(maxsplit))
 
 def rstrip(a, chars=None):
     """
@@ -1333,6 +1282,7 @@ def rstrip(a, chars=None):
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'rstrip', (chars,))
 
+
 def split(a, sep=None, maxsplit=None):
     """
     For each element in `a`, return a list of the words in the
@@ -1366,6 +1316,7 @@ def split(a, sep=None, maxsplit=None):
     return _vec_string(
         a, object_, 'split', [sep] + _clean_args(maxsplit))
 
+
 def splitlines(a, keepends=None):
     """
     For each element in `a`, return a list of the lines in the
@@ -1394,6 +1345,7 @@ def splitlines(a, keepends=None):
     return _vec_string(
         a, object_, 'splitlines', _clean_args(keepends))
 
+
 def startswith(a, prefix, start=0, end=None):
     """
     Returns a boolean array which is `True` where the string element
@@ -1405,7 +1357,7 @@ def startswith(a, prefix, start=0, end=None):
     ----------
     a : array_like of str or unicode
 
-    suffix : str
+    prefix : str
 
     start, end : int, optional
         With optional `start`, test beginning at that position. With
@@ -1424,6 +1376,7 @@ def startswith(a, prefix, start=0, end=None):
     return _vec_string(
         a, bool_, 'startswith', [prefix, start] + _clean_args(end))
 
+
 def strip(a, chars=None):
     """
     For each element in `a`, return a copy with the leading and
@@ -1471,9 +1424,10 @@ def strip(a, chars=None):
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'strip', _clean_args(chars))
 
+
 def swapcase(a):
     """
-    For each element in `a`, return a copy of the string with
+    Return element-wise a copy of the string with
     uppercase characters converted to lowercase and vice versa.
 
     Calls `str.swapcase` element-wise.
@@ -1482,11 +1436,12 @@ def swapcase(a):
 
     Parameters
     ----------
-    a : array-like of str or unicode
+    a : array_like, {str, unicode}
+        Input array.
 
     Returns
     -------
-    out : ndarray
+    out : ndarray, {str, unicode}
         Output array of str or unicode, depending on input type
 
     See also
@@ -1501,14 +1456,17 @@ def swapcase(a):
     >>> np.char.swapcase(c)
     array(['A1b C', '1B cA', 'B cA1', 'Ca1B'],
         dtype='|S5')
+
     """
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'swapcase')
 
+
 def title(a):
     """
-    For each element in `a`, return a titlecased version of the
-    string: words start with uppercase characters, all remaining cased
+    Return element-wise title cased version of string or unicode.
+
+    Title case words start with uppercase characters, all remaining cased
     characters are lowercase.
 
     Calls `str.title` element-wise.
@@ -1517,7 +1475,8 @@ def title(a):
 
     Parameters
     ----------
-    a : array-like of str or unicode
+    a : array_like, {str, unicode}
+        Input array.
 
     Returns
     -------
@@ -1536,10 +1495,12 @@ def title(a):
     >>> np.char.title(c)
     array(['A1B C', '1B Ca', 'B Ca1', 'Ca1B'],
         dtype='|S5')
+
     """
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'title')
 
+
 def translate(a, table, deletechars=None):
     """
     For each element in `a`, return a copy of the string where all
@@ -1575,9 +1536,10 @@ def translate(a, table, deletechars=None):
         return _vec_string(
             a_arr, a_arr.dtype, 'translate', [table] + _clean_args(deletechars))
 
+
 def upper(a):
     """
-    Return an array with the elements of `a` converted to uppercase.
+    Return an array with the elements converted to uppercase.
 
     Calls `str.upper` element-wise.
 
@@ -1585,11 +1547,12 @@ def upper(a):
 
     Parameters
     ----------
-    a : array-like of str or unicode
+    a : array_like, {str, unicode}
+        Input array.
 
     Returns
     -------
-    out : ndarray
+    out : ndarray, {str, unicode}
         Output array of str or unicode, depending on input type
 
     See also
@@ -1604,30 +1567,34 @@ def upper(a):
     >>> np.char.upper(c)
     array(['A1B C', '1BCA', 'BCA1'],
         dtype='|S5')
+
     """
     a_arr = numpy.asarray(a)
     return _vec_string(a_arr, a_arr.dtype, 'upper')
 
+
 def zfill(a, width):
     """
-    Return the numeric string left-filled with zeros in a string of
-    length `width`.
+    Return the numeric string left-filled with zeros
 
     Calls `str.zfill` element-wise.
 
     Parameters
     ----------
-    a : array-like of str or unicode
+    a : array_like, {str, unicode}
+        Input array.
     width : int
+        Width of string to left-fill elements in `a`.
 
     Returns
     -------
-    out : ndarray
+    out : ndarray, {str, unicode}
         Output array of str or unicode, depending on input type
 
     See also
     --------
     str.zfill
+
     """
     a_arr = numpy.asarray(a)
     width_arr = numpy.asarray(width)
@@ -1635,9 +1602,10 @@ def zfill(a, width):
     return _vec_string(
         a_arr, (a_arr.dtype.type, size), 'zfill', (width_arr,))
 
+
 def isnumeric(a):
     """
-    For each element in `a`, return True if there are only numeric
+    For each element, return True if there are only numeric
     characters in the element.
 
     Calls `unicode.isnumeric` element-wise.
@@ -1648,24 +1616,27 @@ def isnumeric(a):
 
     Parameters
     ----------
-    a : array-like of unicode
+    a : array_like, unicode
+        Input array.
 
     Returns
     -------
-    out : ndarray
-        Array of booleans
+    out : ndarray, bool
+        Array of booleans of same shape as `a`.
 
     See also
     --------
     unicode.isnumeric
+
     """
     if _use_unicode(a) != unicode_:
-        raise TypeError, "isnumeric is only available for Unicode strings and arrays"
+        raise TypeError("isnumeric is only available for Unicode strings and arrays")
     return _vec_string(a, bool_, 'isnumeric')
 
+
 def isdecimal(a):
     """
-    For each element in `a`, return True if there are only decimal
+    For each element, return True if there are only decimal
     characters in the element.
 
     Calls `unicode.isdecimal` element-wise.
@@ -1676,19 +1647,21 @@ def isdecimal(a):
 
     Parameters
     ----------
-    a : array-like of unicode
+    a : array_like, unicode
+        Input array.
 
     Returns
     -------
-    out : ndarray
-        Array of booleans
+    out : ndarray, bool
+        Array of booleans identical in shape to `a`.
 
     See also
     --------
     unicode.isdecimal
+
     """
     if _use_unicode(a) != unicode_:
-        raise TypeError, "isnumeric is only available for Unicode strings and arrays"
+        raise TypeError("isnumeric is only available for Unicode strings and arrays")
     return _vec_string(a, bool_, 'isdecimal')
 
 
@@ -1872,11 +1845,11 @@ class chararray(ndarray):
     def __array_finalize__(self, obj):
         # The b is a special case because it is used for reconstructing.
         if not _globalvar and self.dtype.char not in 'SUbc':
-            raise ValueError, "Can only create a chararray from string data."
+            raise ValueError("Can only create a chararray from string data.")
 
     def __getitem__(self, obj):
         val = ndarray.__getitem__(self, obj)
-        if issubclass(val.dtype.type, character):
+        if issubclass(val.dtype.type, character) and not _len(val) == 0:
             temp = val.rstrip()
             if _len(temp) == 0:
                 val = ''
@@ -2041,28 +2014,16 @@ class chararray(ndarray):
         """
         return asarray(capitalize(self))
 
-    if sys.version_info >= (2, 4):
-        def center(self, width, fillchar=' '):
-            """
-            Return a copy of `self` with its elements centered in a
-            string of length `width`.
+    def center(self, width, fillchar=' '):
+        """
+        Return a copy of `self` with its elements centered in a
+        string of length `width`.
 
-            See also
-            --------
-            center
-            """
-            return asarray(center(self, width, fillchar))
-    else:
-        def center(self, width):
-            """
-            Return a copy of `self` with its elements centered in a
-            string of length `width`.
-
-            See also
-            --------
-            center
-            """
-            return asarray(center(self, width))
+        See also
+        --------
+        center
+        """
+        return asarray(center(self, width, fillchar))
 
     def count(self, sub, start=0, end=None):
         """
@@ -2247,29 +2208,17 @@ class chararray(ndarray):
         """
         return join(self, seq)
 
-    if sys.version_info >= (2, 4):
-        def ljust(self, width, fillchar=' '):
-            """
-            Return an array with the elements of `self` left-justified in a
-            string of length `width`.
+    def ljust(self, width, fillchar=' '):
+        """
+        Return an array with the elements of `self` left-justified in a
+        string of length `width`.
 
-            See also
-            --------
-            char.ljust
+        See also
+        --------
+        char.ljust
 
-            """
-            return asarray(ljust(self, width, fillchar))
-    else:
-        def ljust(self, width):
-            """
-            Return an array with the elements of `self` left-justified in a
-            string of length `width`.
-
-            See also
-            --------
-            ljust
-            """
-            return asarray(ljust(self, width))
+        """
+        return asarray(ljust(self, width, fillchar))
 
     def lower(self):
         """
@@ -2295,16 +2244,15 @@ class chararray(ndarray):
         """
         return asarray(lstrip(self, chars))
 
-    if sys.version_info >= (2, 5):
-        def partition(self, sep):
-            """
-            Partition each element in `self` around `sep`.
+    def partition(self, sep):
+        """
+        Partition each element in `self` around `sep`.
 
-            See also
-            --------
-            partition
-            """
-            return asarray(partition(self, sep))
+        See also
+        --------
+        partition
+        """
+        return asarray(partition(self, sep))
 
     def replace(self, old, new, count=None):
         """
@@ -2343,53 +2291,39 @@ class chararray(ndarray):
         """
         return rindex(self, sub, start, end)
 
-    if sys.version_info >= (2, 4):
-        def rjust(self, width, fillchar=' '):
-            """
-            Return an array with the elements of `self`
-            right-justified in a string of length `width`.
+    def rjust(self, width, fillchar=' '):
+        """
+        Return an array with the elements of `self`
+        right-justified in a string of length `width`.
 
-            See also
-            --------
-            char.rjust
+        See also
+        --------
+        char.rjust
 
-            """
-            return asarray(rjust(self, width, fillchar))
-    else:
-        def rjust(self, width):
-            """
-            Return an array with the elements of `self`
-            right-justified in a string of length `width`.
+        """
+        return asarray(rjust(self, width, fillchar))
 
-            See also
-            --------
-            rjust
-            """
-            return asarray(rjust(self, width))
+    def rpartition(self, sep):
+        """
+        Partition each element in `self` around `sep`.
 
-    if sys.version_info >= (2, 5):
-        def rpartition(self, sep):
-            """
-            Partition each element in `self` around `sep`.
+        See also
+        --------
+        rpartition
+        """
+        return asarray(rpartition(self, sep))
 
-            See also
-            --------
-            rpartition
-            """
-            return asarray(rpartition(self, sep))
+    def rsplit(self, sep=None, maxsplit=None):
+        """
+        For each element in `self`, return a list of the words in
+        the string, using `sep` as the delimiter string.
 
-    if sys.version_info >= (2, 4):
-        def rsplit(self, sep=None, maxsplit=None):
-            """
-            For each element in `self`, return a list of the words in
-            the string, using `sep` as the delimiter string.
+        See also
+        --------
+        char.rsplit
 
-            See also
-            --------
-            char.rsplit
-
-            """
-            return rsplit(self, sep, maxsplit)
+        """
+        return rsplit(self, sep, maxsplit)
 
     def rstrip(self, chars=None):
         """
@@ -2659,7 +2593,7 @@ def array(obj, itemsize=None, copy=True, unicode=None, order=None):
             # to divide by the size of a single Unicode character,
             # which for Numpy is always 4
             if issubclass(obj.dtype.type, unicode_):
-                itemsize /= 4
+                itemsize //= 4
 
         if unicode is None:
             if issubclass(obj.dtype.type, unicode_):
