@@ -2,6 +2,7 @@ from __future__ import division, absolute_import, print_function
 
 import sys
 import platform
+import warnings
 
 from numpy.testing import *
 from numpy.testing.utils import _gen_alignment_data
@@ -35,6 +36,36 @@ class TestConstants(TestCase):
     def test_euler_gamma(self):
         assert_allclose(ncu.euler_gamma, 0.5772156649015329, 1e-15)
 
+class TestOut(TestCase):
+    def test_out_subok(self):
+        for b in (True, False):
+            aout = np.array(0.5)
+
+            r = np.add(aout, 2, out=aout)
+            assert_(r is aout)
+            assert_array_equal(r, aout)
+
+            r = np.add(aout, 2, out=aout, subok=b)
+            assert_(r is aout)
+            assert_array_equal(r, aout)
+
+            r = np.add(aout, 2, aout, subok=False)
+            assert_(r is aout)
+            assert_array_equal(r, aout)
+
+            d = np.ones(5)
+            o1 = np.zeros(5)
+            o2 = np.zeros(5, dtype=np.int32)
+            r1, r2 = np.frexp(d, o1, o2, subok=b)
+            assert_(r1 is o1)
+            assert_array_equal(r1, o1)
+            assert_(r2 is o2)
+            assert_array_equal(r2, o2)
+
+            r1, r2 = np.frexp(d, out=o1, subok=b)
+            assert_(r1 is o1)
+            assert_array_equal(r1, o1)
+
 
 class TestDivision(TestCase):
     def test_division_int(self):
@@ -99,14 +130,16 @@ class TestPower(TestCase):
         assert_almost_equal(x**(0.5), [1., ncu.sqrt(2), ncu.sqrt(3)])
 
         for out, inp, msg in _gen_alignment_data(dtype=np.float32,
-                                                 type='unary'):
+                                                 type='unary',
+                                                 max_size=11):
             exp = [ncu.sqrt(i) for i in inp]
             assert_almost_equal(inp**(0.5), exp, err_msg=msg)
             np.sqrt(inp, out=out)
             assert_equal(out, exp, err_msg=msg)
 
         for out, inp, msg in _gen_alignment_data(dtype=np.float64,
-                                                 type='unary'):
+                                                 type='unary',
+                                                 max_size=7):
             exp = [ncu.sqrt(i) for i in inp]
             assert_almost_equal(inp**(0.5), exp, err_msg=msg)
             np.sqrt(inp, out=out)
@@ -187,6 +220,27 @@ class TestLog2(TestCase):
             yf = np.array(y, dtype=dt)
             assert_almost_equal(np.log2(xf), yf)
 
+    def test_log2_ints(self):
+        # a good log2 implementation should provide this,
+        # might fail on OS with bad libm
+        for i in range(1, 65):
+            v = np.log2(2.**i)
+            assert_equal(v, float(i), err_msg='at exponent %d' % i)
+
+    def test_log2_special(self):
+        assert_equal(np.log2(1.), 0.)
+        assert_equal(np.log2(np.inf), np.inf)
+        assert_(np.isnan(np.log2(np.nan)))
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.filterwarnings('always', '', RuntimeWarning)
+            assert_(np.isnan(np.log2(-1.)))
+            assert_(np.isnan(np.log2(-np.inf)))
+            assert_equal(np.log2(0.), -np.inf)
+            assert_(w[0].category is RuntimeWarning)
+            assert_(w[1].category is RuntimeWarning)
+            assert_(w[2].category is RuntimeWarning)
+
 
 class TestExp2(TestCase):
     def test_exp2_values(self) :
@@ -660,6 +714,24 @@ class TestFmin(_FilterInvalids):
             assert_equal(np.fmin(arg1, arg2), out)
 
 
+class TestBool(TestCase):
+    def test_truth_table(self):
+        arg1 = [False, False, True, True]
+        arg2 = [False, True, False, True]
+        # OR
+        out = [False, True, True, True]
+        for func in (np.logical_or, np.bitwise_or, np.maximum):
+            assert_equal(func(arg1, arg2), out)
+        # AND
+        out = [False, False, False, True]
+        for func in (np.logical_and, np.bitwise_and, np.minimum):
+            assert_equal(func(arg1, arg2), out)
+        # XOR
+        out = [False, True, True, False]
+        for func in (np.logical_xor, np.bitwise_xor, np.not_equal):
+            assert_equal(func(arg1, arg2), out)
+
+
 class TestFloatingPoint(TestCase):
     def test_floating_point(self):
         assert_equal(ncu.FLOATING_POINT_SUPPORT, 1)
@@ -693,17 +765,17 @@ class TestSign(TestCase):
 
 class TestMinMax(TestCase):
     def test_minmax_blocked(self):
-        "simd tests on max/min"
-        for dt in [np.float32, np.float64]:
+        # simd tests on max/min, test all alignments, slow but important
+        # for 2 * vz + 2 * (vs - 1) + 1 (unrolled once)
+        for dt, sz in [(np.float32, 15), (np.float64, 7)]:
             for out, inp, msg in _gen_alignment_data(dtype=dt, type='unary',
-                                                     max_size=17):
+                                                     max_size=sz):
                 for i in range(inp.size):
                     inp[:] = np.arange(inp.size, dtype=dt)
                     inp[i] = np.nan
-                    self.assertTrue(np.isnan(inp.max()),
-                                    msg=repr(inp) + '\n' + msg)
-                    self.assertTrue(np.isnan(inp.min()),
-                                    msg=repr(inp) + '\n' + msg)
+                    emsg = lambda: '%r\n%s' % (inp, msg)
+                    assert_(np.isnan(inp.max()), msg=emsg)
+                    assert_(np.isnan(inp.min()), msg=emsg)
 
                     inp[i] = 1e10
                     assert_equal(inp.max(), 1e10, err_msg=msg)
@@ -718,17 +790,21 @@ class TestMinMax(TestCase):
         assert_equal(d.min(), d[0])
 
 
-class TestAbsolute(TestCase):
-    def test_abs_blocked(self):
-        "simd tests on abs"
-        for dt in [np.float32, np.float64]:
+class TestAbsoluteNegative(TestCase):
+    def test_abs_neg_blocked(self):
+        # simd tests on abs, test all alignments for vz + 2 * (vs - 1) + 1
+        for dt, sz in [(np.float32, 11), (np.float64, 5)]:
             for out, inp, msg in _gen_alignment_data(dtype=dt, type='unary',
-                                                     max_size=17):
+                                                     max_size=sz):
                 tgt = [ncu.absolute(i) for i in inp]
                 np.absolute(inp, out=out)
                 assert_equal(out, tgt, err_msg=msg)
                 self.assertTrue((out >= 0).all())
 
+                tgt = [-1*(i) for i in inp]
+                np.negative(inp, out=out)
+                assert_equal(out, tgt, err_msg=msg)
+
                 # will throw invalid flag depending on compiler optimizations
                 with np.errstate(invalid='ignore'):
                     for v in [np.nan, -np.inf, np.inf]:
@@ -915,6 +991,274 @@ class TestSpecialMethods(TestCase):
         assert_equal(ncu.maximum(a, B()), 0)
         assert_equal(ncu.maximum(a, C()), 0)
 
+    @dec.skipif(True) # ufunc override disabled for 1.9
+    def test_ufunc_override(self):
+        class A(object):
+            def __numpy_ufunc__(self, func, method, pos, inputs, **kwargs):
+                return self, func, method, pos, inputs, kwargs
+
+        a = A()
+
+        b = np.matrix([1])
+        c = np.array([1])
+        res0 = np.multiply(a, b)
+        res1 = np.dot(a, b)
+
+        # self
+        assert_equal(res0[0], a)
+        assert_equal(res1[0], a)
+        assert_equal(res0[1], np.multiply)
+        assert_equal(res1[1], np.dot)
+        assert_equal(res0[2], '__call__')
+        assert_equal(res1[2], '__call__')
+        assert_equal(res0[3], 0)
+        assert_equal(res1[3], 0)
+        assert_equal(res0[4], (a, b))
+        assert_equal(res1[4], (a, b))
+        assert_equal(res0[5], {})
+        assert_equal(res1[5], {})
+
+    @dec.skipif(True) # ufunc override disabled for 1.9
+    def test_ufunc_override_mro(self):
+
+        # Some multi arg functions for testing.
+        def tres_mul(a, b, c):
+            return a * b * c
+
+        def quatro_mul(a, b, c, d):
+            return a * b * c * d
+
+        # Make these into ufuncs.
+        three_mul_ufunc = np.frompyfunc(tres_mul, 3, 1)
+        four_mul_ufunc = np.frompyfunc(quatro_mul, 4, 1)
+
+        class A(object):
+            def __numpy_ufunc__(self, func, method, pos, inputs, **kwargs):
+                return "A"
+
+        class ASub(A):
+            def __numpy_ufunc__(self, func, method, pos, inputs, **kwargs):
+                return "ASub"
+
+        class B(object):
+            def __numpy_ufunc__(self, func, method, pos, inputs, **kwargs):
+                return "B"
+
+        class C(object):
+            def __numpy_ufunc__(self, func, method, pos, inputs, **kwargs):
+                return NotImplemented
+
+        class CSub(object):
+            def __numpy_ufunc__(self, func, method, pos, inputs, **kwargs):
+                return NotImplemented
+
+
+
+        a = A()
+        a_sub = ASub()
+        b = B()
+        c = C()
+        c_sub = CSub()
+
+        # Standard
+        res =  np.multiply(a, a_sub)
+        assert_equal(res, "ASub")
+        res = np.multiply(a_sub, b)
+        assert_equal(res, "ASub")
+
+        # With 1 NotImplemented
+        res = np.multiply(c, a)
+        assert_equal(res, "A")
+
+        # Both NotImplemented.
+        assert_raises(TypeError, np.multiply, c, c_sub)
+        assert_raises(TypeError, np.multiply, c_sub, c)
+        assert_raises(TypeError, np.multiply, 2, c)
+
+        # Ternary testing.
+        assert_equal(three_mul_ufunc(a, 1, 2), "A")
+        assert_equal(three_mul_ufunc(1, a, 2), "A")
+        assert_equal(three_mul_ufunc(1, 2, a), "A")
+
+        assert_equal(three_mul_ufunc(a, a, 6), "A")
+        assert_equal(three_mul_ufunc(a, 2, a), "A")
+        assert_equal(three_mul_ufunc(a, 2, b), "A")
+        assert_equal(three_mul_ufunc(a, 2, a_sub), "ASub")
+        assert_equal(three_mul_ufunc(a, a_sub, 3), "ASub")
+        assert_equal(three_mul_ufunc(c, a_sub, 3), "ASub")
+        assert_equal(three_mul_ufunc(1, a_sub, c), "ASub")
+
+        assert_equal(three_mul_ufunc(a, b, c), "A")
+        assert_equal(three_mul_ufunc(a, b, c_sub), "A")
+        assert_equal(three_mul_ufunc(1, 2, b), "B")
+
+        assert_raises(TypeError, three_mul_ufunc, 1, 2, c)
+        assert_raises(TypeError, three_mul_ufunc, c_sub, 2, c)
+        assert_raises(TypeError, three_mul_ufunc, c_sub, 2, 3)
+
+        # Quaternary testing.
+        assert_equal(four_mul_ufunc(a, 1, 2, 3), "A")
+        assert_equal(four_mul_ufunc(1, a, 2, 3), "A")
+        assert_equal(four_mul_ufunc(1, 1, a, 3), "A")
+        assert_equal(four_mul_ufunc(1, 1, 2, a), "A")
+
+        assert_equal(four_mul_ufunc(a, b, 2, 3), "A")
+        assert_equal(four_mul_ufunc(1, a, 2, b), "A")
+        assert_equal(four_mul_ufunc(b, 1, a, 3), "B")
+        assert_equal(four_mul_ufunc(a_sub, 1, 2, a), "ASub")
+        assert_equal(four_mul_ufunc(a, 1, 2, a_sub), "ASub")
+
+        assert_raises(TypeError, four_mul_ufunc, 1, 2, 3, c)
+        assert_raises(TypeError, four_mul_ufunc, 1, 2, c_sub, c)
+        assert_raises(TypeError, four_mul_ufunc, 1, c, c_sub, c)
+
+    @dec.skipif(True) # ufunc override disabled for 1.9
+    def test_ufunc_override_methods(self):
+        class A(object):
+            def __numpy_ufunc__(self, ufunc, method, pos, inputs, **kwargs):
+                return self, ufunc, method, pos, inputs, kwargs
+
+        # __call__
+        a = A()
+        res = np.multiply.__call__(1, a, foo='bar', answer=42)
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], '__call__')
+        assert_equal(res[3], 1)
+        assert_equal(res[4], (1, a))
+        assert_equal(res[5], {'foo': 'bar', 'answer': 42})
+
+        # reduce, positional args
+        res = np.multiply.reduce(a, 'axis0', 'dtype0', 'out0', 'keep0')
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'reduce')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a,))
+        assert_equal(res[5], {'dtype':'dtype0',
+                               'out': 'out0',
+                               'keepdims': 'keep0',
+                               'axis': 'axis0'})
+
+        # reduce, kwargs
+        res = np.multiply.reduce(a, axis='axis0', dtype='dtype0', out='out0',
+                                 keepdims='keep0')
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'reduce')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a,))
+        assert_equal(res[5], {'dtype':'dtype0',
+                               'out': 'out0',
+                               'keepdims': 'keep0',
+                               'axis': 'axis0'})
+
+        # accumulate, pos args
+        res = np.multiply.accumulate(a, 'axis0', 'dtype0', 'out0')
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'accumulate')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a,))
+        assert_equal(res[5], {'dtype':'dtype0',
+                               'out': 'out0',
+                               'axis': 'axis0'})
+
+        # accumulate, kwargs
+        res = np.multiply.accumulate(a, axis='axis0', dtype='dtype0',
+                                     out='out0')
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'accumulate')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a,))
+        assert_equal(res[5], {'dtype':'dtype0',
+                               'out': 'out0',
+                               'axis': 'axis0'})
+
+        # reduceat, pos args
+        res = np.multiply.reduceat(a, [4, 2], 'axis0', 'dtype0', 'out0')
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'reduceat')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a, [4, 2]))
+        assert_equal(res[5], {'dtype':'dtype0',
+                               'out': 'out0',
+                               'axis': 'axis0'})
+
+        # reduceat, kwargs
+        res = np.multiply.reduceat(a, [4, 2], axis='axis0', dtype='dtype0',
+                                   out='out0')
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'reduceat')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a, [4, 2]))
+        assert_equal(res[5], {'dtype':'dtype0',
+                               'out': 'out0',
+                               'axis': 'axis0'})
+
+        # outer
+        res = np.multiply.outer(a, 42)
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'outer')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a, 42))
+        assert_equal(res[5], {})
+
+        # at
+        res = np.multiply.at(a, [4, 2], 'b0')
+        assert_equal(res[0], a)
+        assert_equal(res[1], np.multiply)
+        assert_equal(res[2], 'at')
+        assert_equal(res[3], 0)
+        assert_equal(res[4], (a, [4, 2], 'b0'))
+
+    @dec.skipif(True) # ufunc override disabled for 1.9
+    def test_ufunc_override_out(self):
+        class A(object):
+            def __numpy_ufunc__(self, ufunc, method, pos, inputs, **kwargs):
+                return kwargs
+
+
+        class B(object):
+            def __numpy_ufunc__(self, ufunc, method, pos, inputs, **kwargs):
+                return kwargs
+
+        a = A()
+        b = B()
+        res0 = np.multiply(a, b, 'out_arg')
+        res1 = np.multiply(a, b, out='out_arg')
+        res2 = np.multiply(2, b, 'out_arg')
+        res3 = np.multiply(3, b, out='out_arg')
+        res4 = np.multiply(a, 4, 'out_arg')
+        res5 = np.multiply(a, 5, out='out_arg')
+
+        assert_equal(res0['out'], 'out_arg')
+        assert_equal(res1['out'], 'out_arg')
+        assert_equal(res2['out'], 'out_arg')
+        assert_equal(res3['out'], 'out_arg')
+        assert_equal(res4['out'], 'out_arg')
+        assert_equal(res5['out'], 'out_arg')
+
+        # ufuncs with multiple output modf and frexp.
+        res6 = np.modf(a, 'out0', 'out1')
+        res7 = np.frexp(a, 'out0', 'out1')
+        assert_equal(res6['out'][0], 'out0')
+        assert_equal(res6['out'][1], 'out1')
+        assert_equal(res7['out'][0], 'out0')
+        assert_equal(res7['out'][1], 'out1')
+
+    @dec.skipif(True) # ufunc override disabled for 1.9
+    def test_ufunc_override_exception(self):
+        class A(object):
+            def __numpy_ufunc__(self, *a, **kwargs):
+                raise ValueError("oops")
+        a = A()
+        for func in [np.divide, np.dot]:
+            assert_raises(ValueError, func, a, a)
 
 class TestChoose(TestCase):
     def test_mixed(self):
